#include <GL/glut.h>
#include <GL/gl.h>
#include <particle.h>
#include <texture.h>
#include "cloud.h"

Cloud::Cloud(Texture *newgfx, float nx, float ny, float nz) : Particle()
{
    origx = nx;
    origy = ny;
    origz = nz;
    gfx = newgfx;
    ReInit(origx, origy, origz);
}

Cloud::~Cloud()
{
}

void Cloud::Draw()
{
    glPushMatrix();
    glTranslatef(x, y, z);
    glRotatef(ax, 1.0, 0.0, 0.0);
    glRotatef(ay, 0.0, 1.0, 0.0);
    glRotatef(az, 0.0, 0.0, 1.0);
    glCallList(draw_list);
    glPopMatrix();
}

void Cloud::Calc()
{
    if(forceLink)
        x += forceLink->GetFx();        // Wind only affects x on clouds
    ax += rx;
    ay += ry;
    az += rz;
}

void Cloud::ReInit(float nx, float ny, float nz)
{
    alive = 1;
//    r = g = b = 1.0;
//    a = 0.1;//(float)rand()/(RAND_MAX+1.0);
    x = nx;
    y = ny;
    z = nz;
    size = 1.0;
    rx = 0.0;
    ry = 0.0;
    rz = ((5.0*rand()/(RAND_MAX+1.0))-2.5)/10.0;
    ax = ay = az = 0.0;

    draw_list = glGenLists(1);
    glNewList(draw_list, GL_COMPILE);
    gfx->Select();
    glBegin(GL_QUADS);
    glTexCoord2f(0.0, 0.0); glVertex3f(-size, -size, 0.0);
    glTexCoord2f(1.0, 0.0); glVertex3f(+size, -size, 0.0);
    glTexCoord2f(1.0, 1.0); glVertex3f(+size, +size, 0.0);
    glTexCoord2f(0.0, 1.0); glVertex3f(-size, +size, 0.0);
    glEnd();
    glEndList();
}


/*****************************************************************************\
| The Cloud engine that controls the particles                                |
\*****************************************************************************/
CloudEngine::CloudEngine(int newNrParticles, float sx, float sy, float nx, float ny, float nz) : ParticleEngine(newNrParticles, nx, ny, nz)
{
    gfx = new Texture;
    spread_x = sx;
    spread_y = sy;
    gfx->Load(CLOUD_PIC, NO_ALPHA, FLIP_Y);
    for (int i=0; i<NrParticles; i++)
    {
        float off_x, off_y;
        off_x = (spread_x*rand()/(RAND_MAX+1.0))-(spread_x/2);
        off_y = (spread_y*rand()/(RAND_MAX+1.0))-(spread_y/2);
        ps[i] = new Cloud(gfx, x+off_x, y+off_y, z);
    }
}

CloudEngine::~CloudEngine()
{
    for (int i=0; i<NrParticles; i++)
        delete ps[i];
    delete gfx;
}

void CloudEngine::Draw()
{
    glBlendFunc(GL_ONE, GL_ONE);
    for (int i=0; i<NrParticles; i++)
    {
        if (ps[i]->IsAlive())
            ps[i]->Draw();
    }
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
}

void CloudEngine::Calc()
{
    for (int i=0; i<NrParticles; i++)
    {
        if (ps[i]->IsAlive())
            ps[i]->Calc();
    }
}
