#include <iostream.h>
#include <math.h>
#include "linalg.h"

/*
 * Vertex class
 */
Vertex::Vertex()
{
    x = y = z = 0.0;
    r = g = b = a = 1.0;
}

Vertex::Vertex(Vertex &v)
{
    x = v.x;
    y = v.y;
    z = v.z;
    r = g = b = a = 1.0;
}

Vertex::Vertex(Vertex *v)
{
    x = v->x;
    y = v->y;
    z = v->z;
    r = g = b = a = 1.0;
}

Vertex::Vertex(float newX, float newY, float newZ)
{
    x = newX;
    y = newY;
    z = newZ;
    r = g = b = a = 1.0;
}

Vertex::~Vertex()
{
}

void Vertex::GetPoint(float *X, float *Y, float *Z)
{
    *X = x;
    *Y = y;
    *Z = z;
}

void Vertex::SetPoint(float newX, float newY, float newZ)
{
    x = newX;
    y = newY;
    z = newZ;
}

void Vertex::SetPoint(Vertex &v)
{
    x = v.x;
    y = v.y;
    z = v.z;
}

void Vertex::GetColor(float *R, float *G, float *B, float *A)
{
    *R = r;
    *G = g;
    *B = b;
    *A = a;
}

void Vertex::SetColor(float R, float G, float B, float A)
{
    r = R;
    g = G;
    b = B;
    a = A;
}

/*
 * Vector class
 */
Vector::Vector() : Vertex()
{
}

Vector::Vector(float newX, float newY, float newZ) : Vertex(newX, newY, newZ)
{
}

Vector::Vector(Vertex &v) : Vertex(v)
{
}

Vector::Vector(Vertex *v) : Vertex(v)
{
}

Vector::~Vector()
{
}

float Vector::Length()
{
    return sqrt(x*x + y*y + z*z);
}

void Vector::Normalize()
{
    float l = Length();

    x = x/l;
    y = y/l;
    z = z/l;
}

Vector& Vector::operator+(const Vector &b)
{
    Vector *t = new Vector(x+b.x, y+b.y, z+b.z);
    return *t;
}

void Vector::operator+=(const Vector &b)
{
    x += b.x;
    y += b.y;
    z += b.z;
}

Vector& Vector::operator-(const Vector &b)
{
    Vector *t = new Vector(x-b.x, y-b.y, z-b.z);
    return *t;
}

void Vector::operator-=(const Vector &b)
{
    x -= b.x;
    y -= b.y;
    z -= b.z;
}

void Vector::Scale(Vector &v)
{
    x *= v.x;
    y *= v.y;
    z *= v.z;
}

void Vector::Scale(float f)
{
    x *= f;
    y *= f;
    z *= f;
}

float DotProduct(Vector &a, Vector &b)
{
    return (a.x*b.x + a.y*b.y + a.z*b.z);
}

Vector& CrossProduct(Vector &a, Vector &b)
{
    Vector *temp = new Vector(a.y*b.z - a.z*b.y,
                              a.z*b.x - a.x*b.z,
                              a.x*b.y - a.y*b.x);
    return *temp;
}

float Angle(Vector &a, Vector &b)
{
    return DotProduct(a,b)/(a.Length()*b.Length());
}
