#include <string.h>
#include <math.h>
#include "texture.h"

Texture::Texture()
{
    x_size = y_size = 0;
    data = NULL;
    masked = 0;
    loaded = 0;
}

Texture::~Texture()
{
    if (loaded)
        glDeleteTextures(1,&tnr[0]);
    if (masked)
        glDeleteTextures(1,&tnr[1]);
    if (data)
        delete [] data;
}

int Texture::LoadAndConvert(char *fpic, int trans, int flipxy)
{
    if (loaded)
        return 0;                       // only one texture per object/class

	tempTexture = auxDIBImageLoad(fpic);
	if (!tempTexture)
		return 0;

    x_size = tempTexture->sizeX;
    y_size = tempTexture->sizeY;

    if (trans==NO_ALPHA)
    {
        data = new unsigned char[x_size*y_size*3]; // RGB
        if (!data)
            return 0;
        memcpy(data, tempTexture->data, x_size*y_size*3);
    }
    else
    {
        data = new unsigned char[x_size*y_size*4]; // RGBA
        if (!data)
            return 0;
        /* convert rgb -> rgba */
        for (int a=0; a<x_size*y_size; a++)
        {
            data[a*4+0] = tempTexture->data[a*3+0];
            data[a*4+1] = tempTexture->data[a*3+1];
            data[a*4+2] = tempTexture->data[a*3+2];
            data[a*4+3] = (unsigned char)floor((tempTexture->data[a*3+0]+tempTexture->data[a*3+1]+tempTexture->data[a*3+2])/3.0);
//            data[a*4+3] = ((im->rgb_data[a*3+0]+im->rgb_data[a*3+1]+im->rgb_data[a*3+2])==0)?0:1;
        }
    }
	
    return 1;
}

/*
 * Load texture without mask layer
 */
int Texture::Load(char *fpic, int trans, int flipxy)
{
    int rc;
    if (!(rc = LoadAndConvert(fpic, trans, flipxy)))
        return rc;

    /* Load image into OpenGL driver (video memory) */
    glGenTextures(1, tnr);
    glBindTexture(GL_TEXTURE_2D, tnr[0]);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
    if (trans==NO_ALPHA)
    {
        glTexImage2D(GL_TEXTURE_2D, 0, 3, x_size, y_size, 0, GL_RGB, GL_UNSIGNED_BYTE, data);
        gluBuild2DMipmaps(GL_TEXTURE_2D, 3, x_size, y_size, GL_RGB, GL_UNSIGNED_BYTE, data);
    }
    else
    {
        glTexImage2D(GL_TEXTURE_2D, 0, 4, x_size, y_size, 0, GL_RGBA, GL_UNSIGNED_BYTE, data);
        gluBuild2DMipmaps(GL_TEXTURE_2D, 4, x_size, y_size, GL_RGBA, GL_UNSIGNED_BYTE, data);
    }

    delete [] data;
    data = NULL;
    masked = 0;
    loaded = 1;
    return 1;
}

/*
 * Load texture with matching mask layer
 */
int Texture::Load(char *fpic, char *fmask, int trans, int flipxy)
{
    int rc;
    if (!(rc = LoadAndConvert(fpic, trans, flipxy)))
        return rc;

    /* Load image into OpenGL driver (video memory) */
    /* first image, then mask */
    glGenTextures(2, tnr);
    glBindTexture(GL_TEXTURE_2D, tnr[0]);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
    if (trans==NO_ALPHA)
    {
        glTexImage2D(GL_TEXTURE_2D, 0, 3, x_size, y_size, 0, GL_RGB, GL_UNSIGNED_BYTE, data);
        gluBuild2DMipmaps(GL_TEXTURE_2D, 3, x_size, y_size, GL_RGB, GL_UNSIGNED_BYTE, data);
    }
    else
    {
        glTexImage2D(GL_TEXTURE_2D, 0, 4, x_size, y_size, 0, GL_RGBA, GL_UNSIGNED_BYTE, data);
        gluBuild2DMipmaps(GL_TEXTURE_2D, 4, x_size, y_size, GL_RGBA, GL_UNSIGNED_BYTE, data);
    }
    
	tempTexture = auxDIBImageLoad(fpic);
	if (!tempTexture)
		return 0;
    glBindTexture(GL_TEXTURE_2D, tnr[1]);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
    glTexImage2D(GL_TEXTURE_2D, 0, 3, tempTexture->sizeX, tempTexture->sizeY, 0, GL_RGB, GL_UNSIGNED_BYTE, tempTexture->data);
    gluBuild2DMipmaps(GL_TEXTURE_2D, 3, tempTexture->sizeX, tempTexture->sizeY, GL_RGB, GL_UNSIGNED_BYTE, tempTexture->data);

    delete [] data;
    data = NULL;
    masked = loaded = 1;
    return 1;
}

int Texture::Load(unsigned char *new_data, int alpha, int width, int height)
{
    if ((!new_data) || (loaded))
        return 0;

    switch (alpha) {
    case NO_ALPHA:              // picture only has RGB elements
        data = new unsigned char[width*height*3];
        if (!data)
            return 0;
        memcpy(data,new_data,width*height*3);
        break;
    case USE_ALPHA:             // picture allready has RGBA elements
        data = new unsigned char[width*height*4];
        if (!data)
            return 0;
        memcpy(data,new_data,width*height*4);
        break;
    case MAKE_ALPHA:        	// picture only has RGB but wants to be RGBA
        data = new unsigned char[width*height*4];
        if (!data)
            return 0;
        /* convert rgb -> rgba */
        for (int a=0; a<width*height; a++)
        {
            data[a*4+0] = new_data[a*3+0];
            data[a*4+1] = new_data[a*3+1];
            data[a*4+2] = new_data[a*3+2];
            data[a*4+3] = (unsigned char)floor((new_data[a*3+0]+new_data[a*3+1]+new_data[a*3+2])/3.0);
        }
        break;
    };

    /* Load image into OpenGL driver (video memory) */
    glGenTextures(1, tnr);
    glBindTexture(GL_TEXTURE_2D, tnr[0]);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
    if (alpha==NO_ALPHA)
    {
        glTexImage2D(GL_TEXTURE_2D, 0, 3, width, height, 0, GL_RGB, GL_UNSIGNED_BYTE, data);
        gluBuild2DMipmaps(GL_TEXTURE_2D, 3, width, height, GL_RGB, GL_UNSIGNED_BYTE, data);
    }
    else
    {
        glTexImage2D(GL_TEXTURE_2D, 0, 4, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, data);
        gluBuild2DMipmaps(GL_TEXTURE_2D, 4, width, height, GL_RGBA, GL_UNSIGNED_BYTE, data);
    }

    delete [] data;
    data = NULL;
    masked = 0;
    loaded = 1;
    return 1;
}

int Texture::GetID()
{
    return tnr[0];
}

void Texture::Select()
{
    glBindTexture(GL_TEXTURE_2D, tnr[0]);
}

void Texture::SelectMask()
{
    if (masked)
        glBindTexture(GL_TEXTURE_2D, tnr[1]);
    else
        glBindTexture(GL_TEXTURE_2D, tnr[0]);
}
