#include <GL/glut.h>
#include <GL/gl.h>
#include <particle.h>
#include <texture.h>
#include "smoke.h"

Smoke::Smoke(Texture *newgfx, float nx, float ny, float nz) : Particle()
{
    origx = nx;
    origy = ny;
    origz = nz;
    gfx = newgfx;
    ReInit(origx, origy, origz);
}

Smoke::~Smoke()
{
}

void Smoke::Draw()
{
    glPushMatrix();
    glDisable(GL_DEPTH_TEST);
    glTranslatef(x, y, z);
    glRotatef(ax, 1.0, 0.0, 0.0);
    glRotatef(ay, 0.0, 1.0, 0.0);
    glRotatef(az, 0.0, 0.0, 1.0);
    gfx->Select();
    glColor4f(r, g, b, a);
    glBegin(GL_QUADS);
    glTexCoord2f(0.0, 0.0); glVertex3f(-size, -size, 0.0);
    glTexCoord2f(1.0, 0.0); glVertex3f(+size, -size, 0.0);
    glTexCoord2f(1.0, 1.0); glVertex3f(+size, +size, 0.0);
    glTexCoord2f(0.0, 1.0); glVertex3f(-size, +size, 0.0);
    glEnd();
    glEnable(GL_DEPTH_TEST);
    glPopMatrix();
}

void Smoke::Calc()
{
    a -= alpha_decay;
    if (a<=0.0)
        ReInit(origx, origy, origz);    // actually: alive = 0;
    else
    {
        if (forceLink)
        {
            x += forceLink->GetFx();
            y += forceLink->GetFy();
            z += forceLink->GetFz();
        }
/*        else
        { */
            x += dx;
            y += dy;
            z += dz;
//        }
        size += growth;
        ax += rx;
        ay += ry;
        az += rz;
    }
}

void Smoke::ReInit(float nx, float ny, float nz)
{
    alive = 1;
    r = g = b = a = 1.0;
    alpha_decay = (float)(2.0*rand()/(RAND_MAX+1.0))/10.0;	// sort of time to live
    x = nx;
    y = ny;
    z = nz;
    size = 1.0;         // grows as smoke decays
    growth = (float)(3.0*rand()/(RAND_MAX+1.0))/100.0;		// speed of growth
    dx = ((3.0*rand()/(RAND_MAX+1.0))-1.5)/10.0;
    dy = (3.0*rand()/(RAND_MAX+1.0))/10.0;
    dz = 0.0; //(float)(3.0*rand()/(RAND_MAX+1.0))/10.0;
    rx = 0.0;
    ry = 0.0;
    rz = ((5.0*rand()/(RAND_MAX+1.0))-2.5)/10.0;
    ax = ay = az = 0.0;
}


/*****************************************************************************\
| The Smoke engine that controls the particles                                |
\*****************************************************************************/
SmokeEngine::SmokeEngine(int newNrParticles, float nx, float ny, float nz) : ParticleEngine(newNrParticles, nx, ny, nz)
{
    gfx = new Texture;
    gfx->Load(SMOKE_PIC, MAKE_ALPHA, FLIP_Y);
    for (int i=0; i<NrParticles; i++)
        ps[i] = new Smoke(gfx, x, y, z);
}

SmokeEngine::~SmokeEngine()
{
    for (int i=0; i<NrParticles; i++)
        delete ps[i];
    delete gfx;
}

void SmokeEngine::Draw()
{
    glBlendFunc(GL_SRC_ALPHA, GL_ONE);
    for (int i=0; i<NrParticles; i++)
    {
        if (ps[i]->IsAlive())
            ps[i]->Draw();
    }
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
}

void SmokeEngine::Calc()
{
    for (int i=0; i<NrParticles; i++)
    {
        if (ps[i]->IsAlive())
            ps[i]->Calc();
    }
}
